'''
Created on 11 Mar 2022

@author: shalomshachne
'''
import time

from iserver.EzxMsg import EzxMsg
from iserver.enums.msgenums import State, Side


__counter = 0

closed_states = { 
    State.REJT.value,
    State.CAND.value,
    State.FILLED.value,
    State.EXPIRE.value,
    State.OVRF.value, 
    State.DONE.value
    }


def next_id() -> str:
    '''
    Get next unique ID. Uses a combination of millis time and a counter. Not thread safe.
    '''
    global __counter
    if __counter >= 999:
        __counter = 0
    __counter = __counter + 1
    millis = round(time.time() * 1000)
    return str(millis) + '-' + str(__counter)


def is_closed(order : EzxMsg):
    try:
        return order.state in closed_states
    except: 
        # if wrong kind of object gets here and returns false
        pass

def format_order(order: EzxMsg):
    # Buy IBM 100x165.23 (ROID=1234,ssOrderID=45667)
    side = Side(order.side).name    
    return f'{side} {order.symbol} {order.orderQty}x{order.price: .2f} (ROID={order.routerOrderID}) ({order.event}) (Filled={order.cumQty})'
      






'''
 Generated by EZX Code Generator  
Date: 02/24/22 
'''


from iserver.EzxMsg import EzxMsg, EzxMsgMeta
from iserver.msgs import *
from iserver.enums import api

from .Properties import Properties


class GroupAccount(EzxMsg, metaclass=EzxMsgMeta):

	__block_fields__ = {
		'PRPSBLK' : Field('settings', 'PRPSBLK', noop_decode_func, ), 
	}

	def __init__(self, **kwargs):
		self.id : str = None
		self.displayName : str = None
		self.ownerID : str = None
		self.settings : Properties = None
		self.isDefault : int = None
		self.notes : str = None
		super(GroupAccount, self).__init__(**kwargs)




'''
 Generated by EZX Code Generator  
Date: 02/24/22 
'''


from iserver.EzxMsg import EzxMsg, EzxMsgMeta
from iserver.msgs import *
from iserver.enums import api

from .ExecutionStatus import ExecutionStatus
from .OrderRequestInfo import OrderRequestInfo
from .OrderStatus import OrderStatus


class OrderInfoResponse(EzxMsg, metaclass=EzxMsgMeta):

	__block_fields__ = {
		'EB' : Field('executionList', 'EB', noop_decode_func, update_list_item), 
		'ORB' : Field('orderRequestList', 'ORB', noop_decode_func, update_list_item), 
		'OB' : Field('orderStatusList', 'OB', noop_decode_func, update_list_item), 
	}

	def __init__(self, **kwargs):
		self.myID : str = None
		self.routerOrderID : int = None
		self.executionList : list[ExecutionStatus] = None
		self.orderRequestList : list[OrderRequestInfo] = None
		self.orderStatusList : list[OrderStatus] = None
		super(OrderInfoResponse, self).__init__(**kwargs)
		self.msg_type = api.IserverMsgType.APPLICATION.value
		self.msg_subtype = api.IserverMsgSubType.ORDERINFO_RESPONSE.value




import logging.config
import os
from random import randint
import random
import string
import unittest
import time
from time import sleep


_RAND_STR_LENGTH = 10

my_path = os.path.dirname(__file__)
logging.config.fileConfig(f'{my_path}/logging.ini')

def check_logging():
    return True

def random_string(size = _RAND_STR_LENGTH):
    return "".join(random.choices(string.ascii_letters, k=size))

def random_number(smallest=0, largest=999999):
    return randint(smallest, largest)

def assert_fields_match(expected, actual, test_case: unittest.TestCase):
    attrs1 = set(vars(expected).items())
    attrs2 = set(vars(actual).items())
    missing_from_actual = attrs1 - attrs2
    in_actual_not_in_expected = attrs2 - attrs1
    actual_diffs, expected_diffs = len(missing_from_actual), len(in_actual_not_in_expected)
    error = ""
    if (actual_diffs > 0):
        error = "actual not equal: {0}".format_order(missing_from_actual)
    if (expected_diffs > 0):
        error = error + ", in actual not in expected: {0}".format_order(in_actual_not_in_expected)
    test_case.assertEqual(0, actual_diffs + expected_diffs, error)
    
def wait_for_condition(predicate, max_wait_secs : float = 30, sleep_secs: float = .030):    
    '''
    Wait for specified predicate to be true, until a maximum time has elapsed.
    @param predicate: a function which returns true or false 
    @param max_wait_secs: maximum time to wait until condition is true
    @param sleep_secs: sleep interval while waiting. default=.03. A value of 0 means processor spins until done waiting. 
    '''
    start = time.time()
    while not predicate() and time.time() - start <= max_wait_secs:
        sleep(sleep_secs)
    
    
    


'''
 Generated by EZX Code Generator  
Date: 02/24/22 
'''


from iserver.EzxMsg import EzxMsg, EzxMsgMeta
from iserver.msgs import *
from iserver.enums import api

from .Destination import Destination
from .Settings import Settings
from .Properties import Properties


class OMSSettings(EzxMsg, metaclass=EzxMsgMeta):

	__block_fields__ = {
		'DESTSTRATBL' : Field('destStrategies', 'DESTSTRATBL', noop_decode_func, update_list_item), 
		'SETTBLK' : Field('settings', 'SETTBLK', noop_decode_func, ), 
		'PRPSBLK' : Field('companySettings', 'PRPSBLK', noop_decode_func, ), 
	}

	def __init__(self, **kwargs):
		self.destStrategies : list[Destination] = None
		self.settings : Settings = None
		self.companySettings : Properties = None
		super(OMSSettings, self).__init__(**kwargs)
		self.msg_type = api.IserverMsgType.APPLICATION.value
		self.msg_subtype = api.IserverMsgSubType.OMSSETTINGS.value




'''
Created on 16 Mar 2022
Subclasses of API messages tuned to specific use cases
@author: shalomshachne
'''
from iserver.msgs.OrderRequest import OrderRequest
from iserver.enums.msgenums import MsgType

class CancelOrder(OrderRequest):
    def __init__(self, routerOrderID : int):
        super().__init__()
        self.msgType = MsgType.CANC.value
        self.routerOrderID=routerOrderID

        



'''
Created on 22 Feb 2022

@author: shalomshachne
'''
import unittest

from iserver import parse_to_dict


class Test(unittest.TestCase):

    def test_parse_to_dict_with_good_string(self):
        values = 'key1=1645193523396-2key2=1645193523396-3'
        d = parse_to_dict(values)        
        
        self.assertIsNotNone(d)
        key = 'key1'
        expected = '1645193523396-2'
        actual = d.get(key)
        self.assertEqual(expected, actual, key)
        
        key = 'key2'
        expected = '1645193523396-3'
        actual = d.get(key)
        self.assertEqual(expected, actual, key)
        
    def test_parse_to_dict_specify_pairs_separator(self):
        pair_separator = ','
        values = 'key1=1645193523396-2,key2=1645193523396-3'
        d = parse_to_dict(values, pair_separator)        
        
        self.assertIsNotNone(d)
        key = 'key1'
        expected = '1645193523396-2'
        actual = d.get(key)
        self.assertEqual(expected, actual, key)
        
        key = 'key2'
        expected = '1645193523396-3'
        actual = d.get(key)
        self.assertEqual(expected, actual, key)       
        
    def test_parse_to_dict_specify_name_value_separator(self):
        pair_separator = ','
        name_value_separator = '|'
        values = 'key1|1645193523396-2,key2|1645193523396-3'
        d = parse_to_dict(values, pair_separator, name_value_separator)        
        
        self.assertIsNotNone(d)
        key = 'key1'
        expected = '1645193523396-2'
        actual = d.get(key)
        self.assertEqual(expected, actual, key)
        
        key = 'key2'
        expected = '1645193523396-3'
        actual = d.get(key)
        self.assertEqual(expected, actual, key)        
        
    def test_parse_to_dict_specify_name_value_separator_strips_white_space(self):
        pair_separator = ','
        name_value_separator = '|'
        values = 'key1|1645193523396-2 , key2|1645193523396-3'
        d = parse_to_dict(values, pair_separator, name_value_separator)        
        
        self.assertIsNotNone(d)
        key = 'key1'
        expected = '1645193523396-2'
        actual = d.get(key)
        self.assertEqual(expected, actual, key)
        
        key = 'key2'
        expected = '1645193523396-3'
        actual = d.get(key)
        self.assertEqual(expected, actual, key)        
        
    def test_extra_token(self):
        values = 'key1=1645193523396-2key2=1645193523396-3'
        d = parse_to_dict(values)        
        
        self.assertIsNotNone(d)
        key = 'key1'
        expected = '1645193523396-2'
        actual = d.get(key)
        self.assertEqual(expected, actual, key)
        
        key = 'key2'
        expected = '1645193523396-3'
        actual = d.get(key)
        self.assertEqual(expected, actual, key)               


if __name__ == "__main__":
    # import sys;sys.argv = ['', 'Test.test_parse_to_dict_with_good_string']
    unittest.main()



'''
 Generated by EZX Code Generator  
Date: 02/24/22 
'''


from iserver.EzxMsg import EzxMsg, EzxMsgMeta
from iserver.msgs import *
from iserver.enums import api



class AllocInfo(EzxMsg, metaclass=EzxMsgMeta):


	def __init__(self, **kwargs):
		self.allocInfoID : int = None
		self.legNum : int = None
		self.allocAccount : str = None
		self.allocAvgPx : float = None
		self.allocShares : float = None
		self.allocText : str = None
		self.processCode : str = None
		self.brokerOfCredit : str = None
		self.commType : str = None
		self.commission : float = None
		super(AllocInfo, self).__init__(**kwargs)




'''
 Generated by EZX Code Generator  
Date: 02/24/22 
'''

from enum import Enum, unique

@unique
class MsgOrigin(Enum):
	TWOWAY = 1
	ROUTEDIN = 2
	LOCAL = 3
	UNSOL = 4


@unique
class PairState(Enum):
	Cancelled = "CANC"
	Closed = "CLOSED"
	New = "NEW"
	Open = "OPEN"
	Suspended = "SUSP"


@unique
class IDSource(Enum):
	CUSIP = "1"
	WM_CODE = "101"
	NECIGEF = "102"
	SEDOL = "2"
	QUIK = "3"
	ISIN = "4"
	RIC = "5"
	ISO_CURR_CODE = "6"
	ISO_COUNTRY_CODE = "7"
	EXCHANGE_SYMBOL = "8"
	CONSOLIDATE_TAPE_ASSOC = "9"
	VALOREN = "D"
	SICOVAM = "E"


@unique
class ClientOrderType(Enum):
	CLIENT_ORDER_ONLY = 1
	STAGED_ORDER = 2
	AUTO_STAGED_ORDER = 3
	PROXY = 4


@unique
class ReceivedMethod(Enum):
	ELECTRONIC = "E"
	FAX = "F"
	INSTANT_MESSAGE = "I"
	EMAIL = "L"
	MANUAL = "M"
	PHONE = "P"
	VERBAL = "V"


@unique
class SettlementType(Enum):
	REGULAR = 0
	CASH = 1
	NEXT_DAY = 2
	T_PLUS_2 = 3
	T_PLUS_3 = 4
	T_PLUS_4 = 5
	FUTURE = 6
	WHEN_ISSUED = 7
	SELLERS_OPTION = 8
	T_PLUS_5 = 9


@unique
class CFICode(Enum):
	EQUITY_COMMON_SHARES = "ES"
	OPTION_CALL = "OC"
	OPTION_PUT = "OP"


@unique
class MDUpdateAction(Enum):
	New = "0"
	Change = "1"
	Delete = "2"


@unique
class AdminResult(Enum):
	FAILURE = 0
	SUCCESS = 1


@unique
class MarketDataLogonType(Enum):
	MONEY_MARKET = 1
	BONDS = 2


@unique
class ReportFills(Enum):
	WHEN_FILLED = 1
	PARTIAL_FILLS = 2


@unique
class QuoteState(Enum):
	ACCEPTED = "ACCEPTED"
	ACKED = "ACKED"
	CAND = "CAND"
	FILLED = "FILLED"
	NEW = "NEW"
	PASSED = "PASSED"
	REJT = "REJT"


@unique
class Side(Enum):
	BUY = 1
	BUY_TO_COVER = 101
	SELL = 2
	BUY_MINUS = 3
	SELL_PLUS = 4
	SELL_SHORT = 5
	SELL_SHORT_EXEMPT = 6


@unique
class SecType(Enum):
	COMMON_STOCK = "CS"
	FUTURE = "FUT"
	MUTUAL_FUND = "MF"
	OPTION = "OPT"
	PREFERRED_STOCK = "PS"


@unique
class TradingSessionID(Enum):
	EARLY = "1"
	NORMAL = "2"
	LATE = "3"


@unique
class Events(Enum):
	ACK = "ACK"
	CANA = "CANA"
	CANC = "CANC"
	CAND = "CAND"
	CANE = "CANE"
	CANR = "CANR"
	CORE = "CORE"
	DELETED = "DELETED"
	DONE = "DONE"
	EDTD = "EDTD"
	EXEC = "EXEC"
	EXPIRE = "EXPIRE"
	ORDR = "ORDR"
	PENDING = "PENDING"
	REJT = "REJT"
	REPA = "REPA"
	REPD = "REPD"
	REPL = "REPL"
	REPR = "REPR"
	RESTATED = "RESTATED"
	RESUBMITTED = "RESUBMITTED"
	RPTD = "RPTD"
	STOPPED = "STOPPED"
	SUSPENDED = "SUSPENDED"
	UNKNOWN = "UNKNOWN"
	URPTD = "URPTD"


@unique
class Currency(Enum):
	USD = "USD"


@unique
class TimeInForce(Enum):
	DAY = 0
	GOOD_TILL_CANCEL = 1
	AT_THE_OPEN = 2
	IMMEDIATE_OR_CANCEL = 3
	FILL_OR_KILL = 4
	GOOD_TILL_CROSSING = 5
	GOOD_TILL_DATE = 6
	AT_THE_CLOSE = 7


@unique
class PositionEffect(Enum):
	CLOSE = "C"
	FIFO = "F"
	OPEN = "O"
	ROLLED = "R"


@unique
class ExecType(Enum):
	NEW = 0
	PARTIALLY_FILLED = 1
	PENDING_NEW = 10
	DELETED = 100
	RESUBMITTED = 101
	FIX_ENGINE_EXPIRED = 102
	LASTLOOK_MATCH = 103
	LASTLOOK_TIMEOUT = 104
	LASTLOOK_REJECT = 105
	CALCULATED = 11
	EXPIRED = 12
	RESTATED = 13
	PENDING_REPLACE = 14
	TRADE = 15
	TRADE_CORRECT = 16
	TRADE_CANCEL = 17
	TRADE_STATUS = 18
	FILLED = 2
	DONE_FOR_DAY = 3
	CANCELLED = 4
	REPLACED = 5
	PENDING_CANCEL = 6
	STOPPED = 7
	REJECTED = 8
	SUSPENDED = 9


@unique
class PendReq(Enum):
	CANC = "CANC"
	NEW = "NEW"
	NOPEND = "NOPEND"
	REPL = "REPL"


@unique
class TradeLiquidityIndicator(Enum):
	ADDED_LIQUIDITY = "A"
	COND_ADDITIONAL_LIQUIDITY = "C"
	OTHER = "Q"
	REMOVED_LIQUIDITY = "R"
	ROUTED = "X"


@unique
class SecurityExchange(Enum):
	CBOT = "CBOT"
	CME = "CME"


@unique
class InfoType(Enum):
	ORDERS_ONLY = 1
	EXEC_AND_ORDERS = 2


@unique
class QuoteUpdateType(Enum):
	UPDATE = 1
	DELETE = 2


@unique
class MDEntryType(Enum):
	Bid = "0"
	Offer = "1"
	Trade = "2"
	IndexValue = "3"
	OpeningPrice = "4"
	ClosingPrice = "5"
	SettlementPrice = "6"
	TradingSessionHighPrice = "7"
	TradingSessionLowPrice = "8"
	TradingSessionVwapPrice = "9"
	Imbalance = "A"
	TradeVolume = "B"
	OpenInterest = "C"


@unique
class OpenClose(Enum):
	CLOSE = "C"
	OPEN = "O"


@unique
class ReturnCode(Enum):
	OK = 0
	INVALID_USER = 1
	DB_DOWN = 2
	DUP_LOGIN = 3
	USER_ERROR = 4
	SYSTEM_ERROR = 5


@unique
class PutOrCall(Enum):
	Put = 0
	Call = 1


@unique
class HandlInstr(Enum):
	AUTO_NO_BROKER = 1
	AUTO_BROKER_OK = 2
	MANUAL = 3


@unique
class OrdType(Enum):
	MARKET = "1"
	LIMIT = "2"
	STOP = "3"
	STOP_LIMIT = "4"
	MARKET_ON_CLOSE = "5"
	WITH_OR_WITHOUT = "6"
	LIMIT_OR_BETTER = "7"
	LIMIT_WITH_OR_WITHOUT = "8"
	ON_BASIS = "9"
	ON_CLOSE = "A"
	LIMIT_ON_CLOSE = "B"
	FOREX_MARKET = "C"
	PREVIOUSLY_QUOTED = "D"
	PREVIOUSLY_INDICATED = "E"
	FOREX_LIMIT = "F"
	FOREX_SWAP = "G"
	FOREX_PREVIOUSLY_QUOTED = "H"
	FUNARI = "I"
	PEGGED = "P"


@unique
class LogonType(Enum):
	OPEN = 1
	ALL_FILLS_MSGS_AND_ALL_OPEN_ORDERS = 10
	ALL = 2
	EXES = 3
	SENDONLY = 4
	TESTRECORD = 5
	TESTRUN = 6
	AGGREGATE = 7
	ORDSWITHFILLS = 8
	DROPCOPY = 9


@unique
class WashTradeCheck(Enum):
	NO_PRICE_CHECK = "NPC"
	PRICE_PLUS_DESTINATION = "PDC"
	PRICE_ONLY_CHECK = "POC"


@unique
class MsgType(Enum):
	NEW = 1
	CANC = 2
	REPL = 3
	EDIT = 4
	STATUS = 5
	TRADE_ACCEPT = 6
	TRADE_DECLINE = 7


@unique
class AllocTransType(Enum):
	NEW = 0
	REPLACE = 1
	CANCEL = 2


@unique
class MdOrderSource(Enum):
	Pairs = "P"
	Single = "S"


@unique
class RiskUserType(Enum):
	NOT_SET = -1
	CLEARING_FIRM = 0
	BROKER = 1
	TRADER = 2
	CLEARING_FIRM_BROKER = 3


@unique
class ExecTransType(Enum):
	NEW = 0
	CANCEL = 1
	CORRECT = 2
	STATUS = 3


@unique
class CustomerOrFirm(Enum):
	Customer = 0
	Firm = 1


@unique
class State(Enum):
	ACKED = "ACKED"
	CAND = "CAND"
	DONE = "DONE"
	EXPIRE = "EXPIRE"
	FILLED = "FILLED"
	NEW = "NEW"
	OVRF = "OVRF"
	PAFI = "PAFI"
	REJT = "REJT"


@unique
class Country(Enum):
	US = "US"


@unique
class DestStatus(Enum):
	DOWN = 0
	UP = 1


@unique
class ProxyType(Enum):
	HTTP = 1
	SOCKS4 = 2
	SOCKS4A = 3
	SOCKS5 = 4


@unique
class QuoteType(Enum):
	BID = 0
	OFFER = 1


@unique
class CoveredOrUncovered(Enum):
	Covered = 0
	Uncovered = 1


@unique
class SymbolType(Enum):
	NO_MAP = 0
	REALTICK = 1
	ITS = 2
	BLOOMBERG = 3
	LIQUIDNET = 4
	REUTERS = 5
	LVOL = 6


@unique
class CompanySetting(Enum):
	TRADING_DISABLED = "0"
	TRADING_ENABLED = "1"
	MAX_AGGREGATED_EXECUTE_INT = "AG_EX"
	AGGREGATE_MAX_LOSS_INT = "AMXLS"
	CLIENT_CREDIT_LIMIT_DBL = "CCL"
	CLIENT_CREDIT_USED_DBL = "CCU"
	CREDIT_LIMIT_DBL = "CL"
	CLIENT_AGGREGATE_MAX_LOSS_INT = "CLAMXLS"
	CLIENT_MAX_OPEN_ORDERS_PER_SYMBOL_INT = "CLMOOPS"
	CLIENT_POSITION_LIMIT_INT = "CLPS_LIMIT"
	CLIENT_POSITION_LIMIT_USED_INT = "CLPS_LIMIT_USED"
	CLIENT_TRADING_LOSS = "CLTRDLS"
	DISABLE = "DISABLE"
	ENABLE = "ENABLE"
	ALLOW_SELL_SHORT_BOOL = "EN_SS"
	ALLOW_SELL_SHORT_EXEMP_BOOL = "EN_SSE"
	LOCATES_CSV = "LC"
	MAX_NOTIONAL_INT = "MAX_DOLLAR"
	MAX_NOTIONAL_OPT_INT = "MAX_DOLLAR_OPT"
	MAX_DUPES_CSV = "MAX_DUPES"
	MARKET_DATA_SOURCE_ENABLED_BOOL = "MDS_ON"
	MAX_OPEN_ORDERS_INT = "MOO"
	MAX_OPEN_ORDERS_PER_SYMBOL_INT = "MOOPS"
	MAX_PRICE_DIFF_PCT = "MPP"
	MAX_SHARES_LIMIT_INT = "MS"
	MAX_SHARES_LIMIT_OPT_INT = "MS_OPT"
	MOVEORDER_CSV = "MVORD"
	MAX_SHARES_TRADED_INT = "MXST"
	MAX_NET_TRADE_INT = "NET_EX"
	ON_CLOSE_CSV = "OC"
	ODD_LOT_ON_BOOL = "ODDLT"
	ODD_LOT_SYMBOLS_CSV = "ODLTSYM"
	ORDERS_PER_INTERVAL_INT = "OPI"
	POSITION_CSV = "PS"
	POSITION_LIMIT_INT = "PS_LIMIT"
	POSITION_LIMIT_USED_INT = "PS_LIMIT_USED"
	ADV_PCT_INT = "SHPCT"
	SYMLIMIT_BP_INT = "SLIMIT.BP"
	SYMLIMIT_DEFAULT_BP_INT = "SLIMIT.BPDEF"
	SYMLIMIT_DELETE_SYMBOL = "SLIMIT.DEL"
	SYMLIMIT_DEFAULT_MAXSHARES_INT = "SLIMIT.MAXSHDEF"
	SYMLIMIT_MXST_INT = "SLIMIT.MXST"
	SYMLIMIT_DEFAULT_MXST_INT = "SLIMIT.MXSTDEF"
	SYMLIMIT_MAXSHARES_INT = "SLIMIT.SH"
	STOP_SYMBOL = "STOPSYM"
	SYML_MAX_POS_VAL = "SYMXPSVAL"
	TOTAL_EXECUTED_LIMIT_INT = "TEL"
	TOTAL_OPEN_AND_EXECUTED_LIMIT_INT = "TOEL"
	DAY_TRADING_ON_OFF_CSV = "TRADEONOFF"
	TRADING_LOSS = "TRDLS"
	MAX_UNIQUE_POSITIONS_INT = "UPOS"
	WASH_TRADE_CHECK_CSV = "WASH"





'''
 Generated by EZX Code Generator  
Date: 02/24/22 
'''


from iserver.EzxMsg import EzxMsg, EzxMsgMeta
from iserver.msgs import *
from iserver.enums import api



class AccountInfo(EzxMsg, metaclass=EzxMsgMeta):


	def __init__(self, **kwargs):
		self.accountID : str = None
		self.accountName : str = None
		self.groupID : str = None
		self.rrID : str = None
		self.branchName : str = None
		super(AccountInfo, self).__init__(**kwargs)




'''
Created on Oct 7, 2021

@author: Sgoldberg
'''
from iserver.EzxMsg import EzxMsg
from iserver.msgs import msgmaps
from iserver.msgs.msgmaps import block_tag_collisions


def get_block_message(block_tag : str, parent_msg : EzxMsg) -> EzxMsg :
    '''
    Gets the correct block message object specified by the blockTag. Since the same blockTag could be used
    for different block messages (group in API parlance), make sure to get the block message used by this
    particular parent message.
    
    @param block_tag: the API tag string which identifies the message block
    @param parent_msg: the message which contains the message block.
    @return: a message object found or None
    '''    
   
    if block_tag in block_tag_collisions:
        m = select_block_message_from_collisions(block_tag, parent_msg)    
    else:
        m = msgmaps.msgs_by_block_tag.get(block_tag)
    if m:
        return m()


def select_block_message_from_collisions(block_tag : str, parent_msg : EzxMsg) -> EzxMsg :
    '''
    When the block_field has a name collision, we have to also use the parent/owner message of the field
    to find the right class instance to decode the message.
    '''
    owner_msg = parent_msg.get_block_field_owner(block_tag)
    if owner_msg:
        possible_parents =  block_tag_collisions.get(block_tag)
        return possible_parents.get(owner_msg)


def get_message(msg_subtype : int) -> EzxMsg:
    '''
    Gets a message for the specified message subtype
    
    '''
    try:
        m = msgmaps.msgs_by_subtype[msg_subtype]
        return m()
    except KeyError:
        return  # this is okay. just bad msgType.
    except Exception as e:
        # TODO: replace with logging
        print(f'Exception creating message for subtype={msg_subtype}. e={e}')
        return 




'''
 Generated by EZX Code Generator  
Date: 02/24/22 
'''


from iserver.EzxMsg import EzxMsg, EzxMsgMeta
from iserver.msgs import *
from iserver.enums import api

from .QuoteRecord import QuoteRecord


class Quote(EzxMsg, metaclass=EzxMsgMeta):

	__block_fields__ = {
		'QUOTE' : Field('quoteRecords', 'QUOTE', noop_decode_func, update_list_item), 
	}

	def __init__(self, **kwargs):
		self.iServerTime : float = None
		self.engMsgRcvTime : float = None
		self.quoteDateTime : str = None
		self.quoteRecords : list[QuoteRecord] = None
		super(Quote, self).__init__(**kwargs)
		self.msg_type = api.IserverMsgType.APPLICATION.value
		self.msg_subtype = api.IserverMsgSubType.QUOTE.value




import io


API_HEADER_SIZE = 15

def _all_props_filter(key_value):
    return True

   
def msg_equals(msg1, msg2) -> bool:
    if not isinstance(msg1, type(msg2)):
        return False
    
    for name,value in vars(msg1).items():
        if getattr(msg2, name) != value:
            return False

    return True

def to_bean_string(obj, property_filter=lambda name_value: name_value[1] != None):
    '''
    Print the objects properties. The optional property_filter should be a function which can be used
    to filter 2 element tuples (name, value).  By default, this will filter out blank values.
    @author: Sgoldberg
    '''   
    properties_of_interest = list(filter(property_filter, vars(obj).items()))
    buffer = io.StringIO()
    for name,value in properties_of_interest:
        if buffer.tell() > 0: 
            buffer.write(', ')
        buffer.write(f'{name}={value}')
    return buffer.getvalue()


def set_properties(obj : object, properties : dict):
    '''
    Set object properties from the specified dictionary. No type conversion is done.
    '''
    for name,value in properties.items():
        try:
            setattr(obj, name, value)
        except:
            pass

def parse_to_dict(name_value_pairs: str, pair_separator: str = '\001', name_value_separator : str = '=') -> dict:
    '''
    Parse a string into a dictionary
    @param name_value_pairs: set of paired values where a list of keys and values can be parsed
    @param pair_separator: the delimiter which separates each name value pair, for example ','   
    @param name_value_separator: the delimiter which separates each name and value, for example '=' (default)
    '''
    d = dict()
    # not using generator in dict() constructor to allow for irregularities. For example if the input string has an extra
    # delimiter at the end.
    for name_value in (pairs.split(name_value_separator) for pairs in name_value_pairs.split(pair_separator)):
        if len(name_value) > 1:
            d[name_value[0].strip()] = name_value[1].strip()
            
    return d





'''
 Generated by EZX Code Generator  
Date: 02/24/22 
'''


from iserver.EzxMsg import EzxMsg, EzxMsgMeta
from iserver.msgs import *
from iserver.enums import api

from .GroupAccount import GroupAccount


class ReadGroupAccounts(EzxMsg, metaclass=EzxMsgMeta):

	__block_fields__ = {
		'GRPACC' : Field('groupAccounts', 'GRPACC', noop_decode_func, update_list_item), 
	}

	def __init__(self, **kwargs):
		self.groupAccounts : list[GroupAccount] = None
		super(ReadGroupAccounts, self).__init__(**kwargs)
		self.msg_type = api.IserverMsgType.COMMAND.value
		self.msg_subtype = api.IserverMsgSubType.READ_GROUP_ACCOUNTS.value




'''
 Generated by EZX Code Generator  
Date: 02/24/22 
'''


from iserver.EzxMsg import EzxMsg, EzxMsgMeta
from iserver.msgs import *
from iserver.enums import api

from .TagValueMsg import TagValueMsg
from .TagValueMsg import TagValueMsg


class Strategy(EzxMsg, metaclass=EzxMsgMeta):

	__block_fields__ = {
		'STRATSETTVS' : Field('strategySettingTVS', 'STRATSETTVS', noop_decode_func, ), 
		'STRATDEFTVS' : Field('strategyDefaultTVS', 'STRATDEFTVS', noop_decode_func, ), 
	}

	def __init__(self, **kwargs):
		self.strategyName : str = None
		self.screenID : str = None
		self.strategySettingTVS : TagValueMsg = None
		self.strategyDefaultTVS : TagValueMsg = None
		super(Strategy, self).__init__(**kwargs)




'''
Created on 18 Feb 2022

@author: shalomshachne
'''
import unittest
import iserver
from iserver.msgs.StrategyInfo import StrategyInfo
from iserver.msgs.TagValueMsg import TagValueMsg
from iserver import ezx_msg
from iserver.enums.api import IserverMsgSubType
from iserver.msgs.OrderRequest import OrderRequest
from pip._internal import req


class Test(unittest.TestCase):


    def setUp(self):
        self.strategy_info = StrategyInfo()
        self.order_request = OrderRequest()
        


    def tearDown(self):
        pass


    def testDecode(self):
        encoded = 'SYM=symbol-2B=STGBLKSTGYNM=1645193523396-1STRATINFOID=1B=STRATTVSkey1=1645193523396-2key2=1645193523396-3E=STRATTVSE=STGBLK'
        
        req = ezx_msg.decode_message(IserverMsgSubType.ORDER.value, encoded)
        self.assertIsNotNone(req, "decoded request")
        
        expected = 'symbol-2'
        actual = req.symbol 
        self.assertEqual(expected, actual, "symbol")
        
        info = req.strategyInfo
        self.assertIsNotNone(info, "created strategy info")
        
        msg = 'strategyName'
        expected = '1645193523396-1'
        actual = info.strategyName
        self.assertEqual(expected, actual, msg)
        
        msg = 'strategyInfoID'
        expected = 1
        actual = info.strategyInfoID
        self.assertEqual(expected, actual, msg)
        
        
        msg = 'tagValueMsg'
        tv_msg = info.strategyTVS
        self.assertIsInstance(tv_msg, TagValueMsg, msg)
        
        index1 = encoded.index('key1')
        index2 = encoded.index('E=STRATTVS')
        values = encoded[index1:index2]
        d = iserver.parse_to_dict(values)
        msg = 'dictionaries match'
        self.assertDictEqual(d, tv_msg.tag_values, msg)
        
         
    def testEncodeStrategyBlock(self):
        encoded = 'SYM=symbol-2B=STGBLKSTGYNM=1645193523396-1STRATINFOID=1B=STRATTVSkey1=1645193523396-2key2=1645193523396-3E=STRATTVSE=STGBLK'        
        req = ezx_msg.decode_message(IserverMsgSubType.ORDER.value, encoded)           
        encoded2 = ezx_msg.encode(req)
        print(f'encoded2={encoded2}')
        self.assertEqual(encoded, encoded2, 'encoded matches API message')
        
        req2 = ezx_msg.decode_message(IserverMsgSubType.ORDER.value, encoded2)    
        self.assertEqual(req, req2, 'order requests match')
        
        
        
    def testGetSuperClassFields(self):
        req = OrderRequest()
        
        fields = req.__block_fields__
        self.assertTrue('ORB' in fields)
        self.assertTrue(fields.__contains__('ORB'))
        
    
    def testGetBlockFieldFromSuperClass(self):
        req = OrderRequest()
        name = 'ORB'
        f = req.get_block_field(name)
        self.assertIsNotNone(f, f'got field for {name}')
        
        name = 'STGBLK'
        f = req.get_block_field(name)
        self.assertIsNotNone(f, f'got field for {name}')
        
        


if __name__ == "__main__":
    #import sys;sys.argv = ['', 'Test.testName']
    unittest.main()


'''
 Generated by EZX Code Generator  
Date: 02/24/22 
'''


from iserver.EzxMsg import EzxMsg, EzxMsgMeta
from iserver.msgs import *
from iserver.enums import api



class ParentOrderStatus(EzxMsg, metaclass=EzxMsgMeta):


	def __init__(self, **kwargs):
		self.version : int = None
		self.updatedVersion : int = None
		self.seqNo : int = None
		self.side : int = None
		self.orderQty : float = None
		self.cumQty : float = None
		self.leavesQty : float = None
		self.placedQty : float = None
		self.totExec : float = None
		self.workingQty : float = None
		self.lnWorkingQty : float = None
		self.executedQty : float = None
		self.lnExecutedQty : float = None
		self.locateReqd : str = None
		self.mgrLimitPx : float = None
		self.tdrLimitPx : float = None
		self.benchmark : float = None
		self.estMktImpact : float = None
		self.parentKey : str = None
		self.myID : str = None
		self.state : str = None
		self.event : str = None
		self.orderEntryTime : str = None
		self.traderReceiptTime : str = None
		self.symbol : str = None
		self.trader : str = None
		self.portfolio : str = None
		self.securityType : str = None
		self.ordType : str = None
		self.country : str = None
		self.currency : str = None
		self.exDest : str = None
		self.securityID : str = None
		self.idSource : str = None
		self.timeStamp : str = None
		self.userInfo : str = None
		self.userName : str = None
		self.openClose : str = None
		self.securityExchange : str = None
		super(ParentOrderStatus, self).__init__(**kwargs)




'''
 Generated by EZX Code Generator  
Date: 02/24/22 
'''


from iserver.EzxMsg import EzxMsg, EzxMsgMeta
from iserver.msgs import *
from iserver.enums import api

from .OrderRequestInfo import OrderRequestInfo


class Reject(EzxMsg, metaclass=EzxMsgMeta):

	__block_fields__ = {
		'ORB' : Field('rejectedRequest', 'ORB', noop_decode_func, ), 
	}

	def __init__(self, **kwargs):
		self.REJECT_NO_MEMORY = None
		self.REJECT_DECODE_FAILURE = None
		self.REJECT_UNKNOWN_FIXID = None
		self.REJECT_UNKNOWN_ROID = None
		self.REJECT_UNKNOWN_CLOID = None
		self.REJECT_UNKNOWN_EVENT = None
		self.REJECT_SE_FAILURE = None
		self.REJECT_UPDATE_FAILURE = None
		self.REJECT_FIX_FAILURE = None
		self.REJECT_ENCODE_FAILURE = None
		self.REJECT_SYSTEM_EXCEPTION = None
		self.REJECT_USER_NOT_ACTIVE = None
		self.REJECT_ROUTER_FAILURE = None
		self.REJECT_INVALID_REQUEST = None
		self.REJECT_DATABASE_ERROR = None
		self.REJECT_COMMUNICATION_ERROR = None
		self.REJECT_MISSSING_ORDERID = None
		self.REJECT_FAILED_RISK_CHECK = None
		self.REJECT_BAD_INBOUND_ORDER_HANDLING = None
		self.REJECT_BUYING_POWER_EXCEEDED = None
		self.REJECT_DESTINATION_DOWN = None
		self.REJECT_FAILED_RISK_DUPLICATE_CHECK = None
		self.REJECT_FAILED_RISK_ETB_CHECK = None
		self.REJECT_FAILED_RISK_PENDING_ORDER_CHECK = None
		self.REJECT_FAILED_RISK_AGGREGATE_EXECUTED_CHECK = None
		self.REJECT_FAILED_RISK_SELF_TRADE_CHECK = None
		self.REJECT_FAILED_RISK_ORDER_MARKING_CHECK = None
		self.REJECT_FAILED_RISK_ERRONEOUS_PRICE_CHECK = None
		self.REJECT_FAILED_RISK_INVALID_SIDE_CHECK = None
		self.REJECT_FAILED_RISK_MAX_SHARES_CHECK = None
		self.REJECT_FAILED_RISK_MAX_VALUE_CHECK = None
		self.REJECT_FAILED_RISK_ON_CLOSE_CHECK = None
		self.REJECT_CROSSING_ENGINE_REJECT = None
		self.REJECT_POSITION_LIMIT_REACHED = None
		self.REJECT_IPO_MKT_ORDER = None
		self.REJECT_NOT_ENOUGH_LOCATES = None
		self.REJECT_UNKNOWN_ACCOUNT = None
		self.REJECT_MAX_LOSS_REACHED = None
		self.REJECT_POSITION_OR_OPEN_LIMIT_REACHED = None
		self.REJECT_MAX_ORDERS_PER_SECOND = None
		self.REJECT_MAX_UNIQUE_POSITIONS = None
		self.REJECT_MAX_EXECUTED_LIMIT = None
		self.REJECT_MAX_OPEN_AND_EXECUTED_LIMIT = None
		self.REJECT_ADV_PCT_EXCEEDED = None
		self.REJECT_NYSE_TPA_NO_PARENT_ORDER = None
		self.REJECT_FAILED_RISK_MAX_OPEN_ORDERS_CHECK = None
		self.REJECT_FAILED_TICKSIZEPILOT_PRICE_CHECK = None
		self.REJECT_CLIENT_POSITION_LIMIT_REACHED = None
		self.REJECT_CLIENT_MAX_LOSS_REACHED = None
		self.REJECT_CLIENT_MAX_OPEN_ORDERS_PER_SYMBOL = None
		self.REJECT_EXCEEDS_UNROUTEDQTY = None
		self.REJECT_MAX_SHARES_TRADED_LIMIT = None
		self.REJECT_TRADING_NOT_ALLOWED = None
		self.REJECT_CXL_REPLACE_CLOSED_ORDER = None
		self.REJECT_USER_NOT_ALLOWED_TO_TRADE_SYMBOL = None
		self.msgType : int = None
		self.routerOrderID : int = None
		self.myID : str = None
		self.parentKey : str = None
		self.side : int = None
		self.symbol : str = None
		self.price : float = None
		self.orderQty : float = None
		self.returnCode : int = None
		self.returnDesc : str = None
		self.userName : str = None
		self.state : str = None
		self.leavesQty : float = None
		self.cumQty : float = None
		self.avgPrice : float = None
		self.unroutedQty : float = None
		self.rejectedRequest : OrderRequestInfo = None
		super(Reject, self).__init__(**kwargs)
		self.msg_type = api.IserverMsgType.APPLICATION.value
		self.msg_subtype = api.IserverMsgSubType.REJECT.value




'''
 Generated by EZX Code Generator  
Date: 02/24/22 
'''


from iserver.EzxMsg import EzxMsg, EzxMsgMeta
from iserver.msgs import *
from iserver.enums import api

from .Destination import Destination
from .AccountInfo import AccountInfo
from .RiskProfile import RiskProfile
from .RegisteredRep import RegisteredRep
from .TradeUser import TradeUser
from .Settings import Settings


class LogonResponse(EzxMsg, metaclass=EzxMsgMeta):

	__block_fields__ = {
		'DESTSTRATBL' : Field('destStrategies', 'DESTSTRATBL', noop_decode_func, update_list_item), 
		'ACCTINFO' : Field('accountList', 'ACCTINFO', noop_decode_func, update_list_item), 
		'RISKPROFILE' : Field('riskProfileList', 'RISKPROFILE', noop_decode_func, update_list_item), 
		'REGREP' : Field('rrList', 'REGREP', noop_decode_func, update_list_item), 
		'TRDUSER' : Field('tradeUserList', 'TRDUSER', noop_decode_func, update_list_item), 
		'SETTBLK' : Field('settings', 'SETTBLK', noop_decode_func, ), 
	}

	def __init__(self, **kwargs):
		self.destinations : list[list] = None
		self.destStrategies : list[Destination] = None
		self.accountList : list[AccountInfo] = None
		self.riskProfileList : list[RiskProfile] = None
		self.rrList : list[RegisteredRep] = None
		self.tradeUserList : list[TradeUser] = None
		self.settings : Settings = None
		self.returnCode : int = None
		self.returnDesc : str = None
		self.heartbeatIntervalSecs : int = None
		self.creditLimit : float = None
		self.creditUsed : float = None
		self.positionLimit : int = None
		self.positionLimitUsed : int = None
		self.riskUserType : int = None
		super(LogonResponse, self).__init__(**kwargs)
		self.msg_type = api.IserverMsgType.APPLICATION.value
		self.msg_subtype = api.IserverMsgSubType.LOGONRESPONSE.value




'''
 Generated by EZX Code Generator  
Date: 02/24/22 
'''


from iserver.EzxMsg import EzxMsg, EzxMsgMeta
from iserver.msgs import *
from iserver.enums import api



class FixedIncomeMarketDataIncrementalRefresh(EzxMsg, metaclass=EzxMsgMeta):


	def __init__(self, **kwargs):
		self.msgRecvTime : str = None
		self.providerID : str = None
		self.noOfMDEntries : int = None
		self.mdentryType : str = None
		self.mdentryPrice : float = None
		self.mdentrySize : float = None
		self.mdEntryDate : str = None
		self.mdEntryTime : str = None
		self.mdEntryID : str = None
		self.mdUpdateAction : str = None
		self.minQty : float = None
		self.mdMinIncrementOffer : float = None
		self.mdMinBalanceRemainingOffer : float = None
		self.mdYieldToWorst : float = None
		self.mdYieldToMaturity : float = None
		self.mdMineFlag : str = None
		self.mdExecutionType : str = None
		self.securityIDSource : str = None
		self.securityID : str = None
		self.symbol : str = None
		self.product : int = None
		self.mdentryOriginator : str = None
		self.tagValueBlock : str = None
		self.marketDepth : int = None
		self.settlDate : str = None
		super(FixedIncomeMarketDataIncrementalRefresh, self).__init__(**kwargs)
		self.msg_type = api.IserverMsgType.APPLICATION.value
		self.msg_subtype = api.IserverMsgSubType.FIXEDINCOMEMARKETDATAINCREMENTAL.value




'''
 Generated by EZX Code Generator  
Date: 02/24/22 
'''

#Date: 02/24/22
from iserver.msgs import *
api_fields = {
	'dailyAmountTotal' : Field('dailyAmountTotal','DAILYAMT', float, )	,
	'forceLogout' : Field('forceLogout','FLOGOUT', int, )	,
	'addInstructions' : Field('addInstructions','ADDINST', str, )	,
	'mdEntryID' : Field('mdEntryID','317', str, )	,
	'executedQty' : Field('executedQty','EXECQTY', float, )	,
	'clientID' : Field('clientID','CLNTID', int, )	,
	'seqNo' : Field('seqNo','SEQ', int, )	,
	'allocAvgPx' : Field('allocAvgPx','ALLOCAVGPX', float, )	,
	'msgRecvTime' : Field('msgRecvTime','RCTM', str, )	,
	'orderEntryTime' : Field('orderEntryTime','OETIME', str, )	,
	'coverPrice' : Field('coverPrice','CVPX', float, )	,
	'fixEngineName' : Field('fixEngineName','FIXC', str, )	,
	'creditUsed' : Field('creditUsed','CRUSED', float, )	,
	'destinationState' : Field('destinationState','DSTAT', int, )	,
	'userID' : Field('userID','USERID', str, )	,
	'quoteEntryID' : Field('quoteEntryID','283', str, )	,
	'contraBroker' : Field('contraBroker','CONTRABRKR', str, )	,
	'firm' : Field('firm','FRM', str, )	,
	'itemName' : Field('itemName','ITEMNM', str, )	,
	'password' : Field('password','PWD', str, )	,
	'rejectReason' : Field('rejectReason','REJREASON', str, )	,
	'fixConnectionID' : Field('fixConnectionID','CNID', str, )	,
	'txCount' : Field('txCount','TXCNT', int, )	,
	'parentKey3' : Field('parentKey3','PK3', str, )	,
	'rrName' : Field('rrName','RRNAME', str, )	,
	'parentKey1' : Field('parentKey1','PK1', str, )	,
	'yieldType' : Field('yieldType','YTYPE', str, )	,
	'parentKey2' : Field('parentKey2','PK2', str, )	,
	'state' : Field('state','STATE', str, )	,
	'quoteDateTime' : Field('quoteDateTime','ENGRCV', str, )	,
	'mdMsgID' : Field('mdMsgID','273', str, )	,
	'choiceGroupName' : Field('choiceGroupName','GRPNM', str, )	,
	'messageText' : Field('messageText','TXT', str, )	,
	'ySymbol' : Field('ySymbol','YSYM', str, )	,
	'reqState' : Field('reqState','REQSTATE', str, )	,
	'maturityDay' : Field('maturityDay','MATDAY', int, )	,
	'destinations' : Field('destinations','DEST', str, update_list_item)	,
	'yMdSymbol' : Field('yMdSymbol','YMDSYM', str, )	,
	'idSource' : Field('idSource','IDSR', str, )	,
	'ownerID' : Field('ownerID','OID', str, )	,
	'version' : Field('version','VER', int, )	,
	'orderDelayMs' : Field('orderDelayMs','DELAYMS', int, )	,
	'iServerTime' : Field('iServerTime','JTICKS', float, )	,
	'isDefault' : Field('isDefault','ISDFLT', int, )	,
	'stopPrice' : Field('stopPrice','STPX', float, )	,
	'parentKey' : Field('parentKey','PK', str, )	,
	'yAvgPrice' : Field('yAvgPrice','YAVGPX', float, )	,
	'mdMinBalanceRemainingOffer' : Field('mdMinBalanceRemainingOffer','319', float, )	,
	'securityExchange' : Field('securityExchange','SECEXCH', str, )	,
	'replaceThrottleTime' : Field('replaceThrottleTime','REPMS', float, )	,
	'adminResult' : Field('adminResult','RESULT', int, )	,
	'afterHours' : Field('afterHours','AFTERHRS', str, )	,
	'partyID' : Field('partyID','PRTYID', str, )	,
	'dailySharesTotal' : Field('dailySharesTotal','DAILYSHARES', float, )	,
	'filledQty' : Field('filledQty','FQTY', int, )	,
	'hungQty' : Field('hungQty','HQTY', int, )	,
	'logonType' : Field('logonType','TYPE', int, )	,
	'startDate' : Field('startDate','STRTDT', str, )	,
	'systemID' : Field('systemID','SYSID', str, )	,
	'engMsgRcvTime' : Field('engMsgRcvTime','TTIME', float, )	,
	'msgType' : Field('msgType','MTYP', int, )	,
	'ssOrderID' : Field('ssOrderID','SSID', int, )	,
	'displayName' : Field('displayName','NAME', str, )	,
	'lastShares' : Field('lastShares','LSHRS', float, )	,
	'quoteRespID' : Field('quoteRespID','QRESPID', str, )	,
	'clientCreditUsed' : Field('clientCreditUsed','CCRUSED', float, )	,
	'returnDesc' : Field('returnDesc','RETD', str, )	,
	'quoteType' : Field('quoteType','QTYPE', int, )	,
	'sendingTime' : Field('sendingTime','SNTM', str, )	,
	'symbolsfx' : Field('symbolsfx','SYMSFX', str, )	,
	'shares' : Field('shares','SHARES', int, )	,
	'execInstr' : Field('execInstr','EINST', str, )	,
	'errDesc' : Field('errDesc','ERRD', str, )	,
	'pairsSessionID' : Field('pairsSessionID','PRSESSID', int, )	,
	'locateReqd' : Field('locateReqd','LREQD', str, )	,
	'reported' : Field('reported','REPRT', int, )	,
	'commission' : Field('commission','COMM', float, )	,
	'currency' : Field('currency','CURR', str, )	,
	'xSymbol' : Field('xSymbol','XSYM', str, )	,
	'coveredOrUncovered' : Field('coveredOrUncovered','COVUNCOV', int, )	,
	'mdentryOriginator' : Field('mdentryOriginator','MDOG', str, )	,
	'settlementType' : Field('settlementType','STLMT', int, )	,
	'initialScan' : Field('initialScan','ISCAN', float, )	,
	'tagValueBlock' : Field('tagValueBlock','TV', str, )	,
	'mdentryRefID' : Field('mdentryRefID','284', str, )	,
	'mdMinIncrementOffer' : Field('mdMinIncrementOffer','318', float, )	,
	'mdEntryTime' : Field('mdEntryTime','316', str, )	,
	'marketDepth' : Field('marketDepth','351', int, )	,
	'mdentryType' : Field('mdentryType','274', str, )	,
	'noOfMDEntries' : Field('noOfMDEntries','314', int, )	,
	'ySlipAllowance' : Field('ySlipAllowance','YSLIP', float, )	,
	'accessFee' : Field('accessFee','ACCESSFEE', float, )	,
	'positionLimit' : Field('positionLimit','POSLMT', int, )	,
	'lnExecutedQty' : Field('lnExecutedQty','LNEXECQTY', float, )	,
	'ioiTransType' : Field('ioiTransType','IOITRANSTYPE', str, )	,
	'commType' : Field('commType','COMMTYPE', str, )	,
	'quoteCondition' : Field('quoteCondition','277', str, )	,
	'workingQty' : Field('workingQty','WRKQTY', float, )	,
	'country' : Field('country','CTRY', str, )	,
	'symbol' : Field('symbol','SYM', str, )	,
	'userNum' : Field('userNum','UIDN', int, )	,
	'notes' : Field('notes','NOTES', str, )	,
	'yFilledQty' : Field('yFilledQty','YFQTY', int, )	,
	'accruedInterestAmt' : Field('accruedInterestAmt','ACCRUINT', float, )	,
	'gmtEndOfDay' : Field('gmtEndOfDay','DTGMT', str, )	,
	'xAvgPrice' : Field('xAvgPrice','XAVGPX', float, )	,
	'partyIDSource' : Field('partyIDSource','PRTYIDSRC', str, )	,
	'returnCode' : Field('returnCode','RETC', int, )	,
	'legNum' : Field('legNum','LEGNUM', int, )	,
	'traderReceiptTime' : Field('traderReceiptTime','TRTIME', str, )	,
	'creditLimit' : Field('creditLimit','CRLMT', float, )	,
	'refID' : Field('refID','REFID', str, )	,
	'moveAllowance' : Field('moveAllowance','MVALL', float, )	,
	'basketID' : Field('basketID','BSKI', str, )	,
	'mdentrySize' : Field('mdentrySize','276', float, )	,
	'rule10bMinutes' : Field('rule10bMinutes','TENB', int, )	,
	'onBehalfOfCompID' : Field('onBehalfOfCompID','OBCID', str, )	,
	'maturityMonthYear' : Field('maturityMonthYear','MATMONYR', str, )	,
	'accountType' : Field('accountType','ACCTYPE', str, )	,
	'groupID' : Field('groupID','GRPID', str, )	,
	'linkedInboundOrderID' : Field('linkedInboundOrderID','INBNDORDERID', int, )	,
	'isVisible' : Field('isVisible','ISVISI', int, )	,
	'users' : Field('users','UNAM', str, )	,
	'refAllocID' : Field('refAllocID','REFALLOCID', str, )	,
	'riskID' : Field('riskID','RISKID', str, )	,
	'mdentryPositionNumber' : Field('mdentryPositionNumber','278', int, )	,
	'numCompetitors' : Field('numCompetitors','NUMCOMP', int, )	,
	'expireTime' : Field('expireTime','EXPT', str, )	,
	'originatingUser' : Field('originatingUser','UNAM', str, )	,
	'quoteSource' : Field('quoteSource','QSCRC', str, )	,
	'importFileName' : Field('importFileName','IMPFN', str, )	,
	'securityType' : Field('securityType','SECTYPE', str, )	,
	'overRideGroupRestriction' : Field('overRideGroupRestriction','OVRIDE', int, )	,
	'allocTransType' : Field('allocTransType','ALLOCTRANTYPE', int, )	,
	'transactTime' : Field('transactTime','TTIME', str, )	,
	'optAttribute' : Field('optAttribute','OPTATTR', str, )	,
	'yPriceWeight' : Field('yPriceWeight','YPXWGT', float, )	,
	'allocID' : Field('allocID','ALLOCID', str, )	,
	'clientCreditLimit' : Field('clientCreditLimit','CCRLMT', float, )	,
	'idSourceX' : Field('idSourceX','IDSX', str, )	,
	'idSourceY' : Field('idSourceY','IDSY', str, )	,
	'tdrLimitPx' : Field('tdrLimitPx','TLMTPX', float, )	,
	'sendDestStrategies' : Field('sendDestStrategies','SNDDESTSTRAT', int, )	,
	'creditUsageUpdateSecs' : Field('creditUsageUpdateSecs','CUSECS', int, )	,
	'hangAllowance' : Field('hangAllowance','HGALL', int, )	,
	'destination' : Field('destination','DEST', str, )	,
	'importJobName' : Field('importJobName','IMPJN', str, )	,
	'execBroker' : Field('execBroker','EXBRKR', str, )	,
	'iServerGenerated' : Field('iServerGenerated','IGEN', int, )	,
	'mdentryPrice' : Field('mdentryPrice','275', float, )	,
	'quoteStatus' : Field('quoteStatus','QTSTS', int, )	,
	'ezxClientOrderVersion' : Field('ezxClientOrderVersion','EZXCLIDVERSION', int, )	,
	'strategyInfoID' : Field('strategyInfoID','STRATINFOID', int, )	,
	'maxAmountPerOrder' : Field('maxAmountPerOrder','MAXAMTPERORD', float, )	,
	'mdYieldToMaturity' : Field('mdYieldToMaturity','321', float, )	,
	'execType' : Field('execType','EXTYP', int, )	,
	'receivedMethod' : Field('receivedMethod','RECVMTHD', str, )	,
	'exDest' : Field('exDest','EXDST', str, )	,
	'strategyName' : Field('strategyName','STGYNM', str, )	,
	'queueSize' : Field('queueSize','QUSZ', int, )	,
	'myClearingAccount' : Field('myClearingAccount','MYCLRACCT', str, )	,
	'orderID' : Field('orderID','ORDERID', int, )	,
	'pegDiff' : Field('pegDiff','PEGD', float, )	,
	'userName' : Field('userName','UNAM', str, )	,
	'gtcEndOfDay' : Field('gtcEndOfDay','DTGMT', int, )	,
	'decimalPlaces' : Field('decimalPlaces','DECPLCS', int, )	,
	'processCode' : Field('processCode','PROCCODE', str, )	,
	'providerID' : Field('providerID','PVID', str, )	,
	'maxSharesPerOrder' : Field('maxSharesPerOrder','MAXSHARESPERORD', float, )	,
	'positionEffect' : Field('positionEffect','POSEFFECT', str, )	,
	'minQty' : Field('minQty','MINQ', float, )	,
	'userInfo' : Field('userInfo','UINFO', str, )	,
	'companyName' : Field('companyName','CONAM', str, )	,
	'quoteSize' : Field('quoteSize','QSIZE', float, )	,
	'xText' : Field('xText','XTXT', str, )	,
	'quoteID' : Field('quoteID','QUOTEID', str, )	,
	'importTimeStamp' : Field('importTimeStamp','IMPTS', str, )	,
	'allocText' : Field('allocText','ALLOCTEXT', str, )	,
	'lastMkt' : Field('lastMkt','LMKT', str, )	,
	'accountID' : Field('accountID','ACCTID', str, )	,
	'tradingSession' : Field('tradingSession','TRDSESS', str, )	,
	'price' : Field('price','PX', float, )	,
	'discretionaryOffset' : Field('discretionaryOffset','DISOFF', float, )	,
	'options' : Field('options','OPT', str, )	,
	'orderQty' : Field('orderQty','OQTY', float, )	,
	'text' : Field('text','TXT', str, )	,
	'unroutedQty' : Field('unroutedQty','UNRTDQTY', float, )	,
	'id' : Field('id','ID', str, )	,
	'mdHitLiftSide' : Field('mdHitLiftSide','324', str, )	,
	'timeInForce' : Field('timeInForce','TIF', int, )	,
	'senderSubID' : Field('senderSubID','SSUBID', str, )	,
	'clearingFirm' : Field('clearingFirm','CLRFRM', str, )	,
	'mgrLimitPx' : Field('mgrLimitPx','MLMTPX', float, )	,
	'mdUpdateAction' : Field('mdUpdateAction','285', str, )	,
	'sendDestinationState' : Field('sendDestinationState','DSTAT', int, )	,
	'allocInfoID' : Field('allocInfoID','ALLOCINFOID', int, )	,
	'lnWorkingQty' : Field('lnWorkingQty','LNWRKQTY', float, )	,
	'clientOrderLinkID' : Field('clientOrderLinkID','CLORDLINK', str, )	,
	'benchmark' : Field('benchmark','BMRKPX', float, )	,
	'symbolType' : Field('symbolType','SYMT', int, )	,
	'msgOrigin' : Field('msgOrigin','MSGORIGIN', int, )	,
	'ezxClientOrderID' : Field('ezxClientOrderID','EZXCLID', int, )	,
	'exeTimeStamp' : Field('exeTimeStamp','ETIME', str, )	,
	'futSettDate' : Field('futSettDate','FUTSETTLEMENTDATE', str, )	,
	'status' : Field('status','STAT', int, )	,
	'mdYieldToWorst' : Field('mdYieldToWorst','320', float, )	,
	'clientName' : Field('clientName','CLNTNAME', str, )	,
	'avgPrice' : Field('avgPrice','APX', float, )	,
	'heartbeatIntervalSecs' : Field('heartbeatIntervalSecs','HBSECS', int, )	,
	'increment' : Field('increment','INCR', float, )	,
	'carryFirmSubAcct' : Field('carryFirmSubAcct','CARRYFIRM', str, )	,
	'hedgeScan' : Field('hedgeScan','HSCAN', float, )	,
	'benchmarkCurvePoint' : Field('benchmarkCurvePoint','BNCHCPT', float, )	,
	'mdMkt' : Field('mdMkt','352', str, )	,
	'quoteRank' : Field('quoteRank','QTRK', str, )	,
	'origClientOrderID' : Field('origClientOrderID','ORIGCLID', str, )	,
	'referencePrice' : Field('referencePrice','REFPX', float, )	,
	'openClose' : Field('openClose','OPCL', str, )	,
	'myYield' : Field('myYield','YIELD', float, )	,
	'securityIdX' : Field('securityIdX','SECIDX', str, )	,
	'expireDate' : Field('expireDate','EXP', str, )	,
	'securityIdY' : Field('securityIdY','SECIDY', str, )	,
	'event' : Field('event','EVT', str, )	,
	'targetSubID' : Field('targetSubID','TSUBID', str, )	,
	'clearingAcct' : Field('clearingAcct','CLRACT', str, )	,
	'side' : Field('side','SIDE', int, )	,
	'product' : Field('product','PRD', int, )	,
	'pendReq' : Field('pendReq','PENDREQ', str, )	,
	'sendSettings' : Field('sendSettings','SNDSETS', int, )	,
	'updownName' : Field('updownName','UPDWNNM', str, )	,
	'priceOffset' : Field('priceOffset','DISOFF', float, )	,
	'yHedgeWeight' : Field('yHedgeWeight','YHGWGT', float, )	,
	'leavesQty' : Field('leavesQty','LVS', float, )	,
	'priceType' : Field('priceType','PXT', int, )	,
	'numAllocs' : Field('numAllocs','NUMALLOCS', int, )	,
	'rrID' : Field('rrID','RRID', str, )	,
	'sendPosition' : Field('sendPosition','SENDPSTN', int, )	,
	'timeStamp' : Field('timeStamp','TS', str, )	,
	'screenOptions' : Field('screenOptions','SCROPT', str, )	,
	'estMktImpact' : Field('estMktImpact','EMKTIMP', float, )	,
	'exeVersion' : Field('exeVersion','VER', int, )	,
	'contraTrader' : Field('contraTrader','CONTRATRDR', str, )	,
	'brokerOrderID' : Field('brokerOrderID','BOID', str, )	,
	'lastPrice' : Field('lastPrice','LPX', float, )	,
	'date' : Field('date','DT', str, )	,
	'mdEntryDate' : Field('mdEntryDate','315', str, )	,
	'ssExeID' : Field('ssExeID','EID', int, )	,
	'benchmarkSecurityID' : Field('benchmarkSecurityID','NBCHSECID', str, )	,
	'totTotExec' : Field('totTotExec','TTOTEX', float, )	,
	'clientOrderID' : Field('clientOrderID','CLID', str, )	,
	'securityID' : Field('securityID','SECI', str, )	,
	'engineSeqno' : Field('engineSeqno','ESEQ', int, )	,
	'cfiCode' : Field('cfiCode','CFI', str, )	,
	'crossingPrice' : Field('crossingPrice','CRXPX', float, )	,
	'infoType' : Field('infoType','INFOTYPE', int, )	,
	'omsVersion' : Field('omsVersion','OMSVER', str, )	,
	'clientNameID' : Field('clientNameID','CLIENTNAMEID', int, )	,
	'reportFills' : Field('reportFills','REPFILLS', int, )	,
	'portfolio' : Field('portfolio','PFOL', str, )	,
	'engMsgRecvTime' : Field('engMsgRecvTime','ENGRCV', float, )	,
	'yText' : Field('yText','YTXT', str, )	,
	'destinationGroup' : Field('destinationGroup','DESTGRP', str, )	,
	'ogcKey' : Field('ogcKey','OGC', str, )	,
	'positionLimitUsed' : Field('positionLimitUsed','POSLMTUSED', int, )	,
	'mdMineFlag' : Field('mdMineFlag','322', str, )	,
	'customerOrFirm' : Field('customerOrFirm','CUSTORFIR', int, )	,
	'xMdSymbol' : Field('xMdSymbol','XMDSYM', str, )	,
	'fixEngineState' : Field('fixEngineState','STAT', str, )	,
	'initialValue' : Field('initialValue','INITVAL', float, )	,
	'group' : Field('group','GRP', str, )	,
	'userConnID' : Field('userConnID','UCONNID', int, )	,
	'ecnAccessFee' : Field('ecnAccessFee','ECNAFEE', float, )	,
	'ioiID' : Field('ioiID','BOID', str, )	,
	'strikePx' : Field('strikePx','STRIKEPX', float, )	,
	'originalSide' : Field('originalSide','ORIGSIDE', int, )	,
	'mdBwCoverBid' : Field('mdBwCoverBid','325', float, )	,
	'allocAccount' : Field('allocAccount','ALLOCACCT', str, )	,
	'tradeLiquidityIndicator' : Field('tradeLiquidityIndicator','TRLIQIND', str, )	,
	'icebergQty' : Field('icebergQty','MINQ', int, )	,
	'execTransType' : Field('execTransType','EXTRANS', int, )	,
	'putOrCall' : Field('putOrCall','PUTCALL', int, )	,
	'mdExecutionType' : Field('mdExecutionType','323', str, )	,
	'allocRejCode' : Field('allocRejCode','ALLOCREJCD', int, )	,
	'position' : Field('position','PSTN', int, )	,
	'updatedVersion' : Field('updatedVersion','UVER', int, )	,
	'mdReqID' : Field('mdReqID','282', str, )	,
	'ordType' : Field('ordType','OTYP', str, )	,
	'allocShares' : Field('allocShares','ALLOCSHARES', float, )	,
	'destName' : Field('destName','DESTNM', str, )	,
	'settlDate' : Field('settlDate','353', str, )	,
	'maxDistanceFromMarket' : Field('maxDistanceFromMarket','MAXDIST', float, )	,
	'ratioQty' : Field('ratioQty','RATIOQTY', int, )	,
	'accountName' : Field('accountName','ACCTNAME', str, )	,
	'dateGMT' : Field('dateGMT','DTGMT', str, )	,
	'omsOrdType' : Field('omsOrdType','OMSOTYP', str, )	,
	'itemValue' : Field('itemValue','ITEMVAL', str, )	,
	'totExec' : Field('totExec','TOTEX', float, )	,
	'settleDate' : Field('settleDate','QSTLDATE', str, )	,
	'securityIDSource' : Field('securityIDSource','SSRC', str, )	,
	'mpID' : Field('mpID','MPID', str, )	,
	'ezxMdId' : Field('ezxMdId','EZID', str, )	,
	'pairState' : Field('pairState','PRSTATE', str, )	,
	'xSlipAllowance' : Field('xSlipAllowance','XSLIP', float, )	,
	'routerOrderID' : Field('routerOrderID','ROID', int, )	,
	'conditionExpiry' : Field('conditionExpiry','CONDEXP', str, )	,
	'slipAllowance' : Field('slipAllowance','SLIP', float, )	,
	'totCumQty' : Field('totCumQty','TCUMQ', float, )	,
	'fairValue' : Field('fairValue','FV', float, )	,
	'routerAllocID' : Field('routerAllocID','RAID', int, )	,
	'placedQty' : Field('placedQty','PLCD', float, )	,
	'venueType' : Field('venueType','VENUETYP', str, )	,
	'solicited' : Field('solicited','SOLICIT', int, )	,
	'maxFloor' : Field('maxFloor','MAXF', float, )	,
	'allocStatus' : Field('allocStatus','ALLOCSTAT', str, )	,
	'cumQty' : Field('cumQty','CUMQ', float, )	,
	'dispRange' : Field('dispRange','DISPR', float, )	,
	'trader' : Field('trader','TRDR', str, )	,
	'branchName' : Field('branchName','BRANCHNAME', str, )	,
	'tradeDate' : Field('tradeDate','ALLOCTRDDT', str, )	,
	'destinationID' : Field('destinationID','DESTID', int, )	,
	'riskUserType' : Field('riskUserType','RUTYPE', int, )	,
	'command' : Field('command','CMD', str, )	,
	'afterHoursQty' : Field('afterHoursQty','AFTERHRSQTY', float, )	,
	'spread' : Field('spread','SPRD', float, )	,
	'screenID' : Field('screenID','SCRID', str, )	,
	'myID' : Field('myID','MYID', str, )	,
	'avgPx' : Field('avgPx','AVGPX', float, )	,
	'xFilledQty' : Field('xFilledQty','XFQTY', int, )	,
	'handlInstr' : Field('handlInstr','HDLI', int, )	,
	'partyRole' : Field('partyRole','PRTYROL', str, )	,
	'initialPosition' : Field('initialPosition','INITPSTN', int, )	,
	'brokerOfCredit' : Field('brokerOfCredit','BRKOFCRDT', str, )	,
	'clientOrderType' : Field('clientOrderType','CORDERTYP', int, )	,
	'account' : Field('account','ACCT', str, )	,
	'updateType' : Field('updateType','QCHG', int, )	,
}



'''
Created on 11 Mar 2022

@author: shalomshachne
'''
import inspect
import random
import string

from iserver.enums.msgenums import LogonType, Side, Events
from iserver.net import ConnectionInfo
from iserver.msgs.OrderResponse import OrderResponse


__next_int = 0

def create_connection_info(host='localhost', port=15000, user='aUser', password='aPasswd!', company='MyCompany', logon_type=LogonType.ALL.value):
    info = inspect.getargvalues(inspect.currentframe())
    args = info.locals 
    return ConnectionInfo(**args)
    
def next_int_id():
    global __next_int
    __next_int = __next_int + 1
    return __next_int

def random_symbol(max_chars : int =4) -> str:
    return ''.join(random.choices(string.ascii_uppercase, k=max_chars))

def random_price(min_price : float = .01, max_price : float = 999.99) -> float:
    return round(random.uniform(min_price, max_price), 2)

def random_quantity(min_qty : int = 1, max_qty : int = 5000):
    return random.randint(min_qty, max_qty)

def create_order_response(side = Side.BUY.value, symbol = random_symbol(), orderQty = random_quantity(), price=random_price()):
    return OrderResponse(side=side, symbol=symbol, orderQty=orderQty, price=price, routerOrderID=random_quantity(), event=Events.ORDR.value)

  


'''
 Generated by EZX Code Generator  
Date: 02/24/22 
'''


from iserver.EzxMsg import EzxMsg, EzxMsgMeta
from iserver.msgs import *
from iserver.enums import api
from iserver.msgs.AllocationRequestInfo import AllocationRequestInfo



class AllocationRequest(AllocationRequestInfo, metaclass=EzxMsgMeta):


	def __init__(self, **kwargs):
		super(AllocationRequest, self).__init__(**kwargs)
		self.msg_type = api.IserverMsgType.APPLICATION.value
		self.msg_subtype = api.IserverMsgSubType.ALLOCATION_REQUEST.value




'''
Created on 10 Mar 2022

@author: shalomshachne
'''

import socket
from socketserver import BaseRequestHandler, TCPServer

from threading import Condition
import threading
import unittest

from iserver import ezx_msg
from iserver.net import ConnectionInfo, ApiClient, ClientState,\
    NotLoggedInException
from tests import check_logging, wait_for_condition
import logging
from tests.data import test_data_factory
from iserver.msgs.LogonResponse import LogonResponse
from iserver.enums.msgenums import ReturnCode
from iserver.util import next_id
from iserver.enums.api import IserverMsgSubType
from time import sleep
from unittest.mock import Mock
from iserver.msgs.LogonRequest import LogonRequest

logger=logging.getLogger(__name__)

class TestApiClient(unittest.TestCase):


    DEFAULT_PORT = 15002
    
        
    @classmethod
    def setUpClass(cls):
        # super(TestApiClient, cls).setUpClass()
        # sh = logging.StreamHandler(sys.stdout)
        # logging.root.addHandler(sh)
        check_logging()


    def start_server(self):
        self.server = MockIserver(TestApiClient.DEFAULT_PORT, self.handle)
        self.server.start()

    def setUp(self):
        self.connectInfo = ConnectionInfo(host='localhost', port=TestApiClient.DEFAULT_PORT, company='EZX', user='gnadan', password='gnadan', connect_retry_seconds=.1)        
        self.client = ApiClient(self.connectInfo)
        self.start_server()
        
        self.clientTest = lambda *args : None
        self.last_message = None  # message received by server from the ApiClient
        self.wait_condition = Condition()
        
        self.connect_wait_secs = 3
        
        
    def tearDown(self):
        self.client.stop()
        self.server.stop()        
        
    
    def on_state_change(self, state):
        self.last_state_change = state
        
        
    def handle(self, header, body):
        logger.info(f'got a message. header={header}, body={body}')
        self.wait_condition.acquire()
        self.last_message = (header, body)
        self.wait_condition.notify_all()
        self.wait_condition.release()

    def testNotifiesStateChange(self):
        received = 'untouched'
        
        def printState(state): 
            nonlocal received        
            received = state
            logger.debug('got state change=%s', received)
        
        handler = printState
                   
        self.client.on_state_change = handler
         
        self.client._set_state(ClientState.CONNECTED)
        
        self.assertEqual(ClientState.CONNECTED, received, "called back on state change")
        
    
    def test_state_functions(self):
        self.assertFalse(self.client.is_connected(), 'valid initial state')
        
        self.client._set_state(ClientState.CONNECTED)
        self.assertTrue(self.client.is_connected())
        
        self.client._set_state(ClientState.LOGGED_IN)
        self.assertTrue(self.client.is_connected())
        
    

    def assert_logon_received(self):
        message = self.last_message
        self.assertIsNotNone(message, 'got a message')
        header, _ = message
        sub_type = int(header[8:11].decode('UTF-8'))
        self.assertEqual(IserverMsgSubType.LOGON.value, sub_type, 'received logonRequest')

    def test_start_sends_logon(self):
        self.wait_condition.acquire()
        self.client.start()
        self.wait_condition.wait(self.connect_wait_secs)  # 3 seconds max wait.

        self.assert_logon_received()
 
        
    def test_notifies_logon_response(self):
        self.client.on_state_change = self.on_state_change
        
                
        response = LogonResponse(returnCode=ReturnCode.OK.value)
        api_bytes = ezx_msg.to_api_bytes(response, test_data_factory.random_quantity())
        self.client._handle_message(response.msg_type, response.msg_subtype, api_bytes[15:])
        
        self.assertEqual(ClientState.LOGGED_IN, self.client.state(), 'handled logonresponse correctly = ok')
        self.assertEqual(ClientState.LOGGED_IN, self.last_state_change, 'got notified on logon')
        
        
    def test_logon_failure_shuts_down_client(self):
        counter = 0
        def check_logon_failure_notification(state):
            nonlocal counter
            counter = counter + 1
            if counter == 1:
                self.assertEqual(ClientState.LOGON_FAILURE, state, 'notified of logon failure')
                self.got_state_change = True

        self.client.on_state_change = check_logon_failure_notification
        failure = 'wrong user name!'
        response = LogonResponse(returnCode=ReturnCode.INVALID_USER.value, returnDesc=failure)
        api_bytes = ezx_msg.to_api_bytes(response, test_data_factory.random_quantity())

        self.client._handle_message(response.msg_type, response.msg_subtype, api_bytes[15:])
        
        self.assertEqual(ClientState.STOPPED, self.client.state(), 'handled logonresponse correctly = failure')
        self.assertTrue(self.client.is_stopped(), 'shut down on logon failure')
        self.assertTrue(self.got_state_change)
        
        errcode, desc = self.client.get_logon_failure()
        self.assertEqual(ReturnCode.INVALID_USER.value, errcode)
        self.assertEqual(failure, desc)
                        
    def test_client_retries_connection(self):
        self.test_start_sends_logon()
        logger.info('stopping server to cause ApiClient disconnect')
        self.server.stop()
        self.last_message = None
        self.connect_wait_secs = 10 # not sure how long to wait.
        self.wait_condition.acquire()
        logger.info('restarting server...')        
        self.start_server()
        logger.info('waiting for login...')        
        self.wait_condition.wait(self.connect_wait_secs) 
        
        # should reconnect shortly
        # message = self.last_message
        # self.assertIsNotNone(message, 'reconnected and logged in')
        self.assert_logon_received()          
    
    def test_client_can_start_before_server(self):
        self.server.stop()
        
        wait_for_condition(lambda : not self.server.is_running())
        self.assertFalse(self.server.is_running(), 'valid test state, server is shut down')                
        self.client.start()

        self.wait_condition.acquire()        
        self.start_server()
        self.wait_condition.wait(self.connect_wait_secs)         
        self.assert_logon_received()
 
    def test_write_not_connected(self):
        response = test_data_factory.create_order_response()
        with self.assertRaises(NotLoggedInException):
            self.client.send_message(response)
            
    def test_write_connected_can_send_logon(self):
        self.client._set_state(ClientState.CONNECTED)
        mock_socket = Mock()
        mock_socket.sendall(bytearray())
        self.client._socket = mock_socket
        
        mock_socket.sendall.assert_called_once()
        logon = LogonRequest()
        self.client.send_message(logon)
        
        
    
    def test_calling_start_twice_does_nothing(self):
        mock_monitor = Mock()
        self.client.connection_monitor = mock_monitor
        mock_monitor.start()
        
        self.client.start()
        mock_monitor.start.assert_called_once()
 
            
        
    
 
        
    
class MockIserver(object):
    def __init__(self, port, handler):
        
        class ApiClientTestHandler(BaseRequestHandler):
            def __init__(self, *args):
                self.handler = handler  #function(header,body)
                super().__init__(*args)
                logger.debug('ApiClientTestHandler..__init__(): initialized handler')
                
            def handle(self):
                # parse header
                header = self.request.recv(15)
                if not header:
                    logger.error('error: did not receive enough bytes for an API header!')
                    return 
                length, msg_type, msg_subtype = ezx_msg.parse_header(header)
                logger.info(f'msg length={length}, msg_subtype={msg_subtype}')
                body = self.request.recv(length)
                self.handler(header, body)
                 
        self.port = port
        self.server =  TCPServer(('localhost', port), ApiClientTestHandler, False)
        # enable immediate reuse
        self.server.socket.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
        self.server.server_bind()
        self.server.server_activate()
        logger.info(f'Server listening on: {self.server.server_address}')
        
    def start(self):
        logger.info(f'starting server on port={self.port}')
        t = threading.Thread(target=self.server.serve_forever)
        t.daemon = True
        t.start()
        self.server_thread = t
        logger.info(f'Server started.')
        # self.server.serve_forever()
        
    def is_running(self):
        try:
            return self.server_thread.is_alive()
        except AttributeError:
            pass
        
        
        
    def stop(self):
        logger.info('stopping server')
        self.server.shutdown()
        self.server.server_close()
        logger.info('server stopped')

if __name__ == "__main__":
    #import sys;sys.argv = ['', 'Test.testName']
    unittest.main()


'''
Created on 11 Mar 2022

@author: shalomshachne
'''
import unittest
from iserver import util
from iserver.msgs.OrderResponse import OrderResponse
from iserver.enums.msgenums import State
from iserver.msgs.Reject import Reject
from tests.data import test_data_factory

class Test(unittest.TestCase):

    def test_next_id(self):
        last_id = util.next_id()
        for x in range(1000):
            next_id = util.next_id()
            self.assertNotEqual(last_id, next_id, f'interation={x}')
            last_id = next_id


    def test_is_closed(self):
        response = OrderResponse(state=State.ACKED.value)
        self.assertFalse(util.is_closed(response), response.state)
        
        response.state = State.CAND.value
        self.assertTrue(util.is_closed(response), response.state)
        
        #test wrong kind of message
        reject = Reject()        
        self.assertFalse(util.is_closed(reject), 'no error on bad attribute')
        
    def test_format_order(self):
        order = test_data_factory.create_order_response()
        formatted = util.format_order(order)
        print(f'order={formatted}')
        
        


if __name__ == "__main__":
    #import sys;sys.argv = ['', 'Test.test_next_id']
    unittest.main()


'''
 Generated by EZX Code Generator  
Date: 02/24/22 
'''


from iserver.EzxMsg import EzxMsg, EzxMsgMeta
from iserver.msgs import *
from iserver.enums import api
from iserver.msgs.GroupAccountChange import GroupAccountChange



class DeleteAccount(GroupAccountChange, metaclass=EzxMsgMeta):


	def __init__(self, **kwargs):
		super(DeleteAccount, self).__init__(**kwargs)
		self.msg_type = api.IserverMsgType.COMMAND.value
		self.msg_subtype = api.IserverMsgSubType.DELETE_ACCOUNT.value




'''
Created on Oct 6, 2021

@author: Sgoldberg
'''
import unittest

import iserver
from iserver.msgs import LogonRequest


class Test(unittest.TestCase):


    def test_to_bean_string(self):
        l = LogonRequest()
        l.msg_type = None
        l.msg_subtype = None
        l.seqNo = None
        l.userName = 'gnadan'
        l.password = 'mypass'
        expected = 'password=mypass, userName=gnadan'
        actual = iserver.to_bean_string(l)
        self.assertEqual(expected, actual)
        
        actual = str(l)
        self.assertEqual(expected, actual, "inherited to_string method")

    def test_to_bean_string_all(self):
        l = LogonRequest()
        l.userName = 'gnadan'
        l.password = 'mypass'
        expected = 'password=mypass'
        actual = iserver.to_bean_string(l, lambda x : True)
        self.assertTrue(expected in actual)
        expected = 'companyName=None'
        self.assertTrue(expected in actual)
        
        print("str(logon)=" + actual)
        
        expected = actual
        r = repr(l)
        self.assertEqual(expected, r, 'rpr function returns all properties')
        
        
        
        
#         actual = str(l)
#         self.assertEqual(expected, actual, "inherited to_string method")

if __name__ == "__main__":
    #import sys;sys.argv = ['', 'Test.testName']
    unittest.main()


'''
Created on Sep 27, 2021

@author: Sgoldberg
'''

import socket

HOST = "192.168.1.218"
PORT = 15000


def connect() -> socket.socket :
    s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    
    try:
        s.connect((HOST, PORT))
        print('connected...')
        return s
    except Exception as e:
        print(f'connection failed: {e}')




'''
Created on Oct 21, 2021

@author: Sgoldberg
'''
import unittest
import iserver


class Test(unittest.TestCase):


    def setUp(self):
        pass


    def tearDown(self):
        pass







if __name__ == "__main__":
    #import sys;sys.argv = ['', 'Test.testName']
    unittest.main()


'''
 Generated by EZX Code Generator  
Date: 02/24/22 
'''


from iserver.EzxMsg import EzxMsg, EzxMsgMeta
from iserver.msgs import *
from iserver.enums import api

from .GroupAccount import GroupAccount


class GroupAccountChange(EzxMsg, metaclass=EzxMsgMeta):

	__block_fields__ = {
		'GRPACC' : Field('groupAccount', 'GRPACC', noop_decode_func, ), 
	}

	def __init__(self, **kwargs):
		self.myID : str = None
		self.returnCode : int = None
		self.returnDesc : str = None
		self.groupAccount : GroupAccount = None
		super(GroupAccountChange, self).__init__(**kwargs)




from setuptools import setup, find_packages

setup(
    name='ezx-pyapi',
    version='0.1',
    license='MIT',
    author="EZX Inc.",
    author_email='support@ezxinc.com',
    packages=find_packages('.'),
    package_dir={'': '.'},
    url='https://github.com/gmyrianthous/example-publish-pypi',
    keywords='EZX iServer API trading',
    install_requires=[],
)



'''
Created on Oct 25, 2021

@author: Sgoldberg
'''
import unittest
from iserver import msg_factory
from iserver.msgs import ExecutionStatus
from iserver.msgs.OrderRequest import OrderRequest
from iserver.msgs.StrategyInfo import StrategyInfo


class Test(unittest.TestCase):


    def test_get_block_message(self):
        block = 'EB'
        m = msg_factory.get_block_message(block, None)
        self.assertIsInstance(m, ExecutionStatus)
        
        block = 'unknown'
        self.assertIsNone(msg_factory.get_block_message(block, None))
        
    
    def test_get_block_message_for_name_collision(self):
        block_tag = 'STGBLK'
        req = OrderRequest()
        msg = msg_factory.get_block_message(block_tag, req)
        self.assertIsInstance(msg, StrategyInfo)
        
        



if __name__ == "__main__":
    #import sys;sys.argv = ['', 'Test.test_get_block_message']
    unittest.main()


'''
 Generated by EZX Code Generator  
Date: 02/24/22 
'''


from iserver.EzxMsg import EzxMsg, EzxMsgMeta
from iserver.msgs import *
from iserver.enums import api



class ChangePasswordResponse(EzxMsg, metaclass=EzxMsgMeta):


	def __init__(self, **kwargs):
		self.returnCode : int = None
		self.returnDesc : str = None
		super(ChangePasswordResponse, self).__init__(**kwargs)
		self.msg_type = api.IserverMsgType.APPLICATION.value
		self.msg_subtype = api.IserverMsgSubType.CHANGEPASSWORD_RESPONSE.value




'''
Created on Oct 20, 2021

@author: Sgoldberg
'''
from enum import Enum, auto
import logging
import socket

from iserver import ezx_msg
from iserver.EzxMsg import EzxMsg
from iserver.enums.msgenums import LogonType
from iserver.msgs import LogonRequest
from iserver.msgs import LogonResponse
import iserver
import threading
import time



def connect(host, port) -> socket.socket :
    s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    s.connect((host, port))
    logging.info(f'connected to {host}:{port}')
    return s


def _create_login(info):
    l = LogonRequest()
    l.companyName = info.company
    l.userName = info.user
    l.password = info.password
    l.logonType = info.logon_type
    # to-do handle setting seqno
    l.seqNo = 0
    return l
    
    
def empty_msg_handler(msg_subtype: int, msg: EzxMsg):
    logging.info(f"received msg={msg}")


class NotLoggedInException(Exception):
    def __init__(self, msg : str = 'Not logged into iServer'):
        super().__init__(msg)
    


class ClientState(Enum):
    INITIAL = auto()
    CONNECTING = auto()
    CONNECTED = auto()
    LOGGED_IN = auto()
    LOGON_FAILURE = auto()
    DISCONNECTED = auto()
    STOPPED = auto()



DEFAULT_CONNECT_RETRY_SECONDS = 5
DEFAULT_HEARTBEAT_SECONDS = 15 

class ConnectionInfo(object):
    '''
    Specifies all settings related to connecting to the iServer.
    '''

    def __init__(self, host : str, port : int, company : str, user : str, password :str, 
                 logon_type : int = LogonType.ALL_FILLS_MSGS_AND_ALL_OPEN_ORDERS.value, 
                 connect_retry_seconds : float = DEFAULT_CONNECT_RETRY_SECONDS, heartbeat_seconds : int = DEFAULT_HEARTBEAT_SECONDS):
        '''
        Constructor
        '''        
        self.host = host
        self.port = port
        self.company = company
        self.user = user
        self.password = password
        self.logon_type = logon_type
        self.connect_retry_seconds = connect_retry_seconds
        heartbeat_seconds = min(DEFAULT_HEARTBEAT_SECONDS, heartbeat_seconds)
        self.heartbeat_seconds = heartbeat_seconds        
        
    def __str__(self):
        return iserver.to_bean_string(self)

class ApiClient(object):
    '''
    classdocs
    '''

    def __init__(self, connection_info : ConnectionInfo, msg_handler = empty_msg_handler, state_handler = lambda *args: None):
        '''
        Constructor
        '''
        # TODO: throw exception for None?
        self.connection_info = connection_info
        self._msg_handler = msg_handler
        self._state = ClientState.INITIAL
        self.on_state_change = state_handler
        
        self.connection_monitor = ClientMonitor(self)

        
    def start(self):        
        '''
        Start the connection with iServer
        '''
        
        if self.is_running():
            return
        
        logging.info("start(): starting client")
        
        self.connection_monitor.start()
        

    def _connect(self):
        logging.info("start(): connecting to iServer...")
        try:
            self._socket = connect(self.connection_info.host, self.connection_info.port)
            # if we get here, we're connected
            self._set_state(ClientState.CONNECTED)
            
            logging.info("start(): connected")
            self.__recv_thread = threading.Thread(target=self._recv)
            self.__recv_thread.start()
            
            self._sendLogin()
            
        except Exception as e:
            logging.error('start(): failed to connect! ex=%s', e)            
            #todo: save exception?
            
        
    def stop(self):
        '''
        Disconnect and shut down the ApiClient
        '''
        # set STOPPED state first to prevent reconnect logic
        self._set_state(ClientState.STOPPED)        
        try:
            if self._socket:
                self._socket.close()
                self._socket = None
                
                logging.info("stop(): socket closed.")
            
            if self.connection_monitor:
                self.connection_monitor.stop()
                
               
        except:
            pass
         
    def state(self) -> ClientState:
        return self._state
    
    def _set_state(self, state : ClientState):
        logging.debug("ApiClient.state=%s", state)        
        self._state = state
        self.on_state_change(state)
    
    def is_running(self):
        '''
        Indicates that client has been started and is running        
        '''
        return self.connection_monitor.is_running() # if the monitor is running, the client is alive
    
        
    def is_stopped(self):
        '''
        Returns True when Client was stopped programmatically to shut it down.
        '''
        return self._state == ClientState.STOPPED
    
    def is_connected(self):
        return self._state == ClientState.LOGGED_IN or self._state == ClientState.CONNECTED    
    
    def is_loggedin(self):
        return ClientState.LOGGED_IN == self.state()
    
    def _sendLogin(self):
        logging.info("sendLogin(): sending logon message now.")
        self.send_message(_create_login(self.connection_info))
        
    def _handle_message(self, msg_type, msg_subtype, body : bytearray):
        body = body.decode('UTF-8')
        logging.debug(f"_handle_message(): processing subtype={msg_subtype}, msg={body}")
        msg_object = ezx_msg.decode_message(msg_subtype, body)
                
        if msg_object:
            # handle the logon response first.
            if type(msg_object) is LogonResponse:  
                self.handle_logon_response(msg_object)
            # pass message to the user            
            self._msg_handler(msg_subtype, msg_object)
        else:                        
            logging.warn(f"_handle_message(): did not decode subtype={msg_subtype}")         
        
    def handle_logon_response(self, logon_response):
        if logon_response.returnCode == 0:
            self._set_state(ClientState.LOGGED_IN)
        else:
            logging.fatal(f"handle_logon(): logon failed! returnCode={logon_response.returnCode}, failure message={logon_response.returnDesc} ")            
            logging.fatal('handle_logon(): stopping client. Please contact EZX support!')
            self.logon_failure = (logon_response.returnCode, logon_response.returnDesc)
            self._set_state(ClientState.LOGON_FAILURE)
   
            self.stop()
        
    def _recv(self):
        if not self._socket:
            logging.error("_recv(): no socket to receive, exiting")
            return 
        
        # s = self._socket
        with self._socket as s:
            while s:
                try:
                    logging.debug('_recv(): waiting for message...')
                    header = s.recv(15)
                    if not header:
                        break
                    length, msg_type, msg_subtype = ezx_msg.parse_header(header)
                    
                    logging.debug('_recv(): got header. reading body of length=%d', length)                    
                    body = s.recv(length)
                    if not body:
                        break
                    
                    logging.debug('_recv(): processing msg type=%d. bytes received=%s', msg_type, body)                            
                    self._handle_message(msg_type, msg_subtype, body)
                    
                except ConnectionAbortedError:
                    #TODO: figure out whether this was user disconnect or from another source.
                    logging.info("_recv(): socket was disconnected")
                    break
                    
                except Exception as e:
                    logging.error(f"_recv(): error receiving message! ex={e}")
                    break;

        logging.debug("_recv(): no longer receiving messages.")

        self._socket = None

        if not self.is_stopped():
            self._set_state(ClientState.DISCONNECTED)
         
    def send_message(self, api_msg_object: EzxMsg):
        if not self.is_loggedin() and not type(api_msg_object) == LogonRequest: # LogonRequest is sent before being in the logged in state
            raise NotLoggedInException()
        
        try:
            msg = ezx_msg.to_api_bytes(api_msg_object)
            self._socket.sendall(msg)
            logging.debug("send_message(): sent msg=%s", msg)
            
        except Exception as e:
            logging.error('send_message(): error on send - probably a disconnect! ex=%s', e)
    
    
         
    
    def get_logon_failure(self) -> (int, str):
        '''
        Retrieve logon failure code and description if iServer indicates that the Logon failed
        @return: tuple of errcode (int) and message 
        '''
        try:
            return self.logon_failure
        
        except AttributeError: # this might not be set
            pass
            
       
        
class ClientMonitor(object):
    
    
    def __init__(self, client, wait_seconds = DEFAULT_CONNECT_RETRY_SECONDS, heartbeat_seconds = DEFAULT_HEARTBEAT_SECONDS):
        self.client = client
        self.monitor_thread = None


    def retry_seconds(self) -> float: 
        return self.client.connection_info.connect_retry_seconds
    
    def start(self):
        if self.monitor_thread:
            logging.warn('started called when ClientMonitor is already running.')
            return 

        logging.info("ClientMonitor.start(): starting monitor thread")
        self.run = True        
        self.monitor_thread = threading.Thread(target=self.monitor)
        self.monitor_thread.daemon = True                        
        self.monitor_thread.start()
    
        
    def stop(self):
        logging.info("ClientMonitor.stop(): shutting down monitor")
        self.run = False
                        
        
    def monitor(self):
        logging.info("ClientMonitor.monitor(): starting connection monitor")        
        # 1. if not connected, connect
        # 2. if logged in, send heartbeat
        while self.run:
            if self.connect_required():
                self.client._connect()
                time.sleep(self.retry_seconds())            
            
            
            #elif Logged in, send heartbeat.
        
        logging.info('ClientMonitor.monitor(): exiting')
    
    def connect_required(self):
        return self.client.state() == ClientState.INITIAL or self.client.state() == ClientState.DISCONNECTED
    
    def is_running(self):
        if self.monitor_thread:
            return self.monitor_thread.is_alive()
        



class Field:
    '''
    Field descriptor for API message fields.  API messages are sent as strings of name/value pairs. The name is referred to as the API tag.
    
    Attributes:
        api_tag (str): The API tag used for this field. For example:  DEST (for the destination field)
        from_string (function): a function which can convert the string representation of the value to the correct data type. Used for decoding received messages. Default= str
        update_func (function): a function to update the corresponding attribute in the message with the field value. Default = setattr. The other function used is msgs.update_list_item
            if the message attribute is a list object.
    '''

    
    def __init__(self, name=None, api_tag=None, decode_func=str, update_func=setattr):
        '''
        Constructor 
        
        Parameters:
            name (str): Field name/attribute name in the message class
            api_tag (str): the API tag string which identifies the field in the message
            
        
        '''
        self.name = name
        self.api_tag = api_tag
        self.from_string = decode_func
        self.update_func = update_func
                
    def __str__(self):
        return f'Field: name={self.name}, api_tag={self.api_tag}'
    
    def decode_value(self, value):
        return self.from_string(value)
    
    def update_msg_field(self, msg_obj, value):
        # setattr(m, msg_field.name, value)
        self.update_func(msg_obj, self.name, self.decode_value(value))


def noop_decode_func(value):
    '''
        Placeholder function for fields that don't have a decode logic
    '''    
    return value


def update_list_item(obj, property_name, item_value):
    '''
    Fields which are lists of objects use this as their update function.  This will add the specified item_value to the list specified by property_name.
    '''        
    l = getattr(obj, property_name)
    if not l:
        l = []
        setattr(obj, property_name, l)    
    l.append(item_value)    


__all__ = [ 'Field','noop_decode_func' ,'update_list_item' ]


from .ClientOrderResponse import ClientOrderResponse
from .ClientOrderStatus import ClientOrderStatus
from .ExecutionStatus import ExecutionStatus
from .LogonRequest import LogonRequest
from .LogonResponse import LogonResponse
from .OrderResponse import OrderResponse
from .OrderStatus import OrderStatus






'''
 Generated by EZX Code Generator  
Date: 02/24/22 
'''


from iserver.EzxMsg import EzxMsg, EzxMsgMeta
from iserver.msgs import *
from iserver.enums import api



class RiskProfile(EzxMsg, metaclass=EzxMsgMeta):


	def __init__(self, **kwargs):
		self.riskID : str = None
		self.maxAmountPerOrder : float = None
		self.maxSharesPerOrder : float = None
		self.dailyAmountTotal : float = None
		self.dailySharesTotal : float = None
		self.afterHoursQty : float = None
		self.afterHours : str = None
		super(RiskProfile, self).__init__(**kwargs)




'''
 Generated by EZX Code Generator  
Date: 02/24/22 
'''


from iserver.EzxMsg import EzxMsg, EzxMsgMeta
from iserver.msgs import *
from iserver.enums import api



class ClientOrderRequestInfo(EzxMsg, metaclass=EzxMsgMeta):


	def __init__(self, **kwargs):
		self.notes : str = None
		self.addInstructions : str = None
		self.clientNameID : int = None
		self.traderReceiptTime : str = None
		self.orderEntryTime : str = None
		self.msgType : int = None
		self.orderID : int = None
		self.side : int = None
		self.orderQty : float = None
		self.price : float = None
		self.symbol : str = None
		self.symbolsfx : str = None
		self.receivedMethod : str = None
		self.handlInstr : int = None
		self.version : int = None
		self.settlementType : int = None
		self.timeInForce : int = None
		self.stopPrice : float = None
		self.ordType : str = None
		self.idSource : str = None
		self.account : str = None
		self.clientOrderID : str = None
		self.coveredOrUncovered : int = None
		self.customerOrFirm : int = None
		self.currency : str = None
		self.destination : str = None
		self.destinationGroup : str = None
		self.exDest : str = None
		self.execBroker : str = None
		self.execInstr : str = None
		self.expireDate : str = None
		self.expireTime : str = None
		self.locateReqd : str = None
		self.optAttribute : str = None
		self.parentKey1 : str = None
		self.securityID : str = None
		self.securityType : str = None
		self.symbolType : int = None
		self.tagValueBlock : str = None
		self.text : str = None
		self.userInfo : str = None
		self.senderSubID : str = None
		self.targetSubID : str = None
		self.trader : str = None
		self.accountType : str = None
		self.importFileName : str = None
		self.importTimeStamp : str = None
		self.importJobName : str = None
		self.pegDiff : float = None
		self.discretionaryOffset : float = None
		self.minQty : float = None
		self.maxFloor : float = None
		self.dispRange : float = None
		self.timeStamp : str = None
		self.maturityMonthYear : str = None
		self.maturityDay : int = None
		self.clearingAcct : str = None
		self.carryFirmSubAcct : str = None
		self.conditionExpiry : str = None
		self.putOrCall : int = None
		self.strikePx : float = None
		self.mpID : str = None
		self.clientOrderLinkID : str = None
		self.securityExchange : str = None
		self.openClose : str = None
		self.tradingSession : str = None
		self.routerOrderID : int = None
		self.clientOrderType : int = None
		self.solicited : int = None
		self.startDate : str = None
		self.reportFills : int = None
		self.linkedInboundOrderID : int = None
		self.futSettDate : str = None
		self.basketID : str = None
		self.rule10bMinutes : int = None
		self.myID : str = None
		self.referencePrice : float = None
		self.crossingPrice : float = None
		super(ClientOrderRequestInfo, self).__init__(**kwargs)

